@@ -2,7 +2,8 @@
 
 from __future__ import absolute_import
 import logging
-from pyprotobuf.codegenerator import CodeGenerator
+from pyprotobuf.compilerpass import CompilerPass
+from pyprotobuf.codegenerator import FrameVisitor
 
 logger = logging.getLogger('pyprotobuf.dependencysorter')
 
@@ -55,7 +56,7 @@ def TopologicallySorted(graph, get_edges):
             if node.parent != neighbor.parent:
                 continue
 
-            logger.debug('Node %s requires %s', node, neighbor)
+            logger.debug('%s %s requires %s', node.__class__.__name__, node.name, neighbor.name)
             Visit(neighbor)
 
         visiting.remove(node)
@@ -73,22 +74,22 @@ def sort_nodes(targets):
     return TopologicallySorted(targets, get_edges)
 
 import pprint
-class DependencySorter(CodeGenerator):
+class DependencySorter(CompilerPass):
     """ Parse the file looking to link together ParseNodes when they refer to each other.
     """
-
-    types = ['bool', 'string', 'int64', 'int32', 'uint64', 'uint32', 'sint64',
-    'sint32', 'fixed64', 'fixed32', 'sfixed64', 'sfixed32', 'double', 'float',
-    'bytes', 'enum', 'message', 'group']
-
     logger = logging.getLogger('pyprotobuf.NameResolver')
 
-    def visit_FileNode(self, node):
-        # sort the top level messages/enums
-        node.children = list(reversed(sort_nodes(node.children)))
+    def process(self, root):
+        visitor = self.Visitor()
+        visitor.visit(root)
+
+    class Visitor(FrameVisitor):
+        def visit_FileNode(self, node):
+            # sort the top level messages/enums
+            node.children = list(reversed(sort_nodes(node.children)))
 
 
-    def visit_MessageNode(self, node):
-        # sort the top level messages/enums
-        node.children = list(reversed(sort_nodes(node.children)))
+        def visit_MessageNode(self, node):
+            # sort the top level messages/enums
+            node.children = list(reversed(sort_nodes(node.children)))
 
