@@ -4,22 +4,24 @@ import pkg_resources
 import os
 import logging
 import argparse
-
-
 from pyprotobuf.compiler import Compiler
 
-import pyprotobuf.generators.closurelib
-import pyprotobuf.generators.protorpc
-import pyprotobuf.generators.externs
 
 ENTRYPOINT = 'pyprotobuf.generators'
 
 GENERATORS = {
-    'closure': pyprotobuf.generators.closurelib,
-    'python': pyprotobuf.generators.protorpc,
-    'externs':  pyprotobuf.generators.externs
+    'closure': 'pyprotobuf.generators.closurelib',
+    'go': 'pyprotobuf.generators.go',
+    'python': 'pyprotobuf.generators.protorpc',
+    'externs':  'pyprotobuf.generators.externs'
 }
 
+def import_generator(module_name):
+    try:
+        module = __import__(module_name, globals(), locals(), ['__generator__'], -1)
+        return module.__generator__
+    except ImportError:
+        return None
 
 def main(args=None):
     
@@ -36,13 +38,13 @@ def main(args=None):
          module = entrypoint.load()
          generators[entrypoint.name] = getattr(module, '__generator__', None)
 
-
     generator_names = generators.keys()
     
     if args is None:
         parser = argparse.ArgumentParser()
-        parser.add_argument('--format', choices=generator_names, dest='format', default='externs')
+        parser.add_argument('--format', '-f', choices=generator_names, dest='format')
         parser.add_argument('--debug', '-d', default=False, action='store_true')
+        parser.add_argument('--generator', default=None, help="Manually specify a generator module.")
         parser.add_argument('paths', nargs="+")
         args = parser.parse_args()
 
@@ -52,18 +54,35 @@ def main(args=None):
         level = logging.DEBUG
 
     logging.basicConfig(level=level)
-    format = args.format
-        
-    if format not in generators:
-        raise Exception('Wrong format! Expected  {0} Got:'.format(', '.join(generator_names)))    
-    
-    generator_class = generators[format]
-    
+
+
+    overrride_generator_name = getattr(args, 'generator', None)
+
+    if overrride_generator_name is not None:
+        generator_name = overrride_generator_name
+    else:
+        # check built-in generators
+        generator_name = args.format
+
+        if generator_name not in GENERATORS:
+            print ('Wrong format! Expected  {0} Got:'.format(', '.join(generator_names)))
+            sys.exit(-1)
+
+        generator_name = GENERATORS[generator_name]
+
+    generator_class = import_generator(generator_name)
+
+    if generator_class is None:
+        raise Exception("Invalid generator: %s" % generator_name)
+
     out = []
     for path in args.paths:
        path = os.path.abspath(path)
        string = open(path).read()
-       out.append(parse_and_generate(string, path, generator_class))
+
+       compiler = Compiler(generator_class)
+       compiler.generate_header = True
+       out.append(compiler.compile_string(path, string))
     
     return ''.join(out)
     
