@@ -1,3 +1,63 @@
+
+
+class Types(object):
+    BOOL = 'bool'
+    STRING = 'string'
+    INT32 = 'int32'
+    INT64 = 'int64'
+    UINT32 = 'uint32'
+    UINT64 = 'uint64'
+    SINT32 = 'sint32'
+    SINT64 = 'sint64'
+    FIXED32 = 'fixed32'
+    FIXED64 = 'fixed64'
+    SFIXED32 = 'sfixed32'
+    SFIXED64 = 'sfixed64'
+    DOUBLE = 'double'
+    FLOAT = 'float'
+    BYTES = 'bytes'
+    ENUM = 'enum'
+    MESSAGE = 'messsage'
+    GROUP = 'group'
+
+TYPES = [
+    Types.BOOL,
+    Types.STRING,
+    Types.INT32,
+    Types.INT64,
+    Types.UINT32,
+    Types.UINT64,
+    Types.SINT32,
+    Types.SINT64,
+    Types.FIXED32,
+    Types.FIXED64,
+    Types.SFIXED32,
+    Types.SFIXED64,
+    Types.DOUBLE,
+    Types.FLOAT,
+    Types.BYTES,
+    Types.ENUM,
+    Types.MESSAGE,
+    Types.GROUP
+]
+
+NUMERIC_TYPES = [
+    Types.INT32,
+    Types.INT64,
+    Types.UINT32,
+    Types.UINT64,
+    Types.SINT32,
+    Types.SINT64,
+    Types.FIXED32,
+    Types.FIXED64,
+    Types.SFIXED32,
+    Types.SFIXED64,
+    Types.DOUBLE,
+    Types.FLOAT
+]
+
+
+
 class ParseNode(object):
     children = None
     comment = None
@@ -12,6 +72,9 @@ class ParseNode(object):
         # dependencies used for topological sorting
         self.dependencies = set()
 
+        # a map used to store options keys/values
+        self.options = {}
+
     def get_file(self):
         for parent in self.get_parents():
             if isinstance(parent, FileNode):
@@ -26,44 +89,60 @@ class ParseNode(object):
         return self.children[index]
 
     def get_full_typename(self):
-        fqn = [self.type.name]
+        fqn = [str(self.name) if self.name else '']
         parent = self.parent
         while parent is not None:
             if parent.name is not None:
-                fqn.append(parent.name)
+                fqn.append(str(parent.name))
             parent = parent.parent
 
         return '.'.join(list(reversed(fqn)))
 
     def add_dependency(self, dep):
+        """
+            :param dep: ParseNode
+        """
         self.dependencies.add(dep)
         for parent in self.get_parents():
             parent.add_dependency(dep)
 
     def get_dependencies(self):
+        """
+            :rtype: list[ParseNode]
+        """
         return self.dependencies
 
     def get_parents(self):
+        """
+            :rtype: list[ParseNode]
+        """
         parent = self.parent
         while parent is not None:
             yield parent
             parent = parent.parent
 
     def get_full_name(self):
-        fqn = [self.name]
+        assert self.name is not None, ("Name must not be none", self)
+        fqn = [str(self.name)]
         parent = self.parent
         while parent is not None:
             if parent.name is not None:
-                fqn.append(parent.name)
+                fqn.append(str(parent.name))
             parent = parent.parent
 
         return '.'.join(list(reversed(fqn)))
 
-    def get_children_of_type(self, type):
-        return filter(lambda x: isinstance(x, type), self.children)
+    def get_children_of_type(self, node_class):
+        """
+            :type node_class: T
+            :rtype: list[V <= T]
+        """
+        return filter(lambda x: isinstance(x, node_class), self.children)
 
     def __repr__(self):
-        return '%s(%s, %s)' % (
+        if isinstance(self, CommentNode):
+            return '#'
+        return '%s("%s", %s)' % (
             self.__class__.__name__, self.name, self.children)#','.join([repr(f) for f in self.children]))
 
     @property
@@ -75,6 +154,23 @@ class ParseNode(object):
             parent = parent.parent
         return depth
 
+    def resolve_name(self, name):
+        for child in self.children:
+            if child.name == name:
+                return child
+        return None
+
+    def has_option(self, name):
+        return name in self.options
+
+    def set_option(self, name, value):
+        self.options[name] = value
+
+    def get_option(self, *args):
+        return self.options.get(*args)
+
+    def to_proto(self):
+        raise NotImplementedError
 
 import pprint
 
@@ -87,6 +183,9 @@ class RootNode(ParseNode):
 class FileNode(ParseNode):
     filename = None
 
+    def __repr__(self):
+        return 'FileNode(filename="%s")' % self.filename
+
     def get_imports(self):
         return self.get_children_of_type(ImportNode)
 
@@ -109,7 +208,7 @@ class Package(ParseNode):
 class PackageDefinition(ParseNode):
     name = None
 
-    def __str__(self):
+    def to_proto(self):
         return 'package %s;' % self.name
 
     def __repr__(self):
@@ -119,7 +218,7 @@ class PackageDefinition(ParseNode):
 class ServiceNode(ParseNode):
     name = None
 
-    def __str__(self):
+    def to_proto(self):
         string = []
         for child in self.children:
             if isinstance(child, MessageNode):
@@ -154,17 +253,32 @@ class MessageNode(ParseNode):
         string = '\n'.join([('   ' * depth + f) for f in string])
         return "message %s {\n%s\n%s}" % (self.name, string, '   ' * (depth - 1))
 
-    def __str__(self):
+    def to_proto(self):
         return self.tostr(1)
 
 
 class FieldDescriptorNode(ParseNode):
+    """
+        Properties:
+            label: One of "repeated", "optional" or "required".
+            number: The tag/number/id of the field.
+            name: The name of the field.
+            type: Can be a string (Enum of proto types), MessageNode or EnumNode.
+    """
     label = None
     number = None
     name = None
     type = None
 
-    def __str__(self):
+    class LabelType(object):
+        REPEATED = 'repeated'
+        OPTIONAL = 'optional'
+        REQUIRED = 'required'
+
+    def __repr__(self):
+        return 'FieldDescriptorNode(number="%s", name="%s", label="%s", type="%s")' % (self.number, self.name,
+                                                                                    self.label, self.type)
+    def to_proto(self):
         t = self.type
         if isinstance(self.type, MessageNode):
             t = self.type.name
@@ -172,6 +286,14 @@ class FieldDescriptorNode(ParseNode):
 
 
 class EnumNode(ParseNode):
+    name = None
+
+    def __repr__(self):
+        map = {}
+        for c in self.children:
+            map[str(c.name)] = str(c.value)
+
+        return 'EnumNode("%s", %s)' % (self.name, map)
 
     def has(self, key):
         for c in self.children:
@@ -187,32 +309,57 @@ class EnumNode(ParseNode):
 
         return None
 
-    def __str__(self):
+    def to_proto(self):
         return 'enum %s { %s }' % (self.name, ' '.join(map(str, self.children)) )
 
 
 class EnumAssignmentNode(ParseNode):
-    def __str__(self):
+    name = None
+    value = None
+    def to_proto(self):
         return '{0} = {1};'.format(self.name, self.value)
 
 
 class OptionNode(ParseNode):
-    def __str__(self):
+    name = None
+    value = None
+    def to_proto(self):
         return 'option {0} = {1};'.format(self.name, self.value)
 
 
 class ExtendNode(ParseNode):
-    def __str__(self):
+    name = None
+    def __init__(self, parent=None):
+        """
+            :type message_node: MessageNode
+        """
+        super(ExtendNode, self).__init__(parent)
+
+    @property
+    def message_node(self):
+        """
+            :rtype: MessageNode
+        """
+        return self._message_node
+
+    @message_node.setter
+    def message_node(self, v):
+        """
+            :type v: MessageNode
+        """
+        self._message_node = v
+
+    def to_proto(self):
         return 'extend %s { %s }' % (self.name, ' '.join(map(str, self.children)) )
 
 
 class OptionalNode(ParseNode):
-    def __str__(self):
+    def to_proto(self):
         return 'optional {0} {1} = {2};'.format(self.name, self.type, self.value)
 
 
 class SyntaxNode(ParseNode):
-    def __str__(self):
+    def to_proto(self):
         return 'syntax = "{0}";'.format(self.value)
 
 
@@ -221,7 +368,7 @@ class ImportNode(ParseNode):
     """
     value = None
 
-    def __str__(self):
+    def to_proto(self):
         return 'import "{0}";'.format(self.value)
 
     def __repr__(self):
@@ -229,6 +376,5 @@ class ImportNode(ParseNode):
 
 
 class ExtensionsNode(ParseNode):
-    def __str__(self):
+    def to_proto(self):
         return 'extension  {0} to {1};'.format(self.start, self.end)
-
