@@ -68,18 +68,6 @@ import logging
 logger = logging.getLogger('pyprotobuf.parser')
 
 
-def identifier(scanner, token): return TOKEN_NAME, token
-
-
-def operator(scanner, token):   return "OPERATOR", token
-
-
-def digit(scanner, token):      return "DIGIT", token
-
-
-def end_stmnt(scanner, token):  return "END_STATEMENT"
-
-
 scanner = re.Scanner([
     (r"[a-zA-Z_][\.\w]*", lambda scanner, token: (TOKEN_NAME, token)),
     (r"//.*", lambda scanner, token: (TOKEN_COMMENT, token[2:])),
@@ -160,6 +148,9 @@ class ProtoParser(object):
 
     def read_single_line_comment(self, t):
         comment = []
+        node = CommentNode()
+        node.line, node.column = t.line, t.column
+
         while t.has_next():
             token = t.next_token(False)
             if token.value == "\n":
@@ -167,17 +158,30 @@ class ProtoParser(object):
                 break
             comment.append(token.value)
         logger.debug('Read comment %s', ' '.join(comment))
-        node = CommentNode()
+
         node.value = ' '.join(comment)
         return node
 
     def read_multi_line_comment(self, t):
+        node = CommentNode()
+        node.line = t.line
+        node.column = t.column
+
         star = t.next_token()
         assert star.value == '*'
         comment = []
+        i = 0
         while t.has_next():
-            token = t.next_token()
+
+            token = t.next_token(skipnl=False)
+
+            if i > 0 and token.value != '\n':
+                comment.append(' ')
+
+            i += 1
+
             if token.value == "\n":
+                i = 0
                 t.line += 1
 
             if token.value == '*' and t.readahead().value == '/':
@@ -186,13 +190,14 @@ class ProtoParser(object):
                 break
 
             comment.append(token.value)
+
         logger.debug('Read comment %s', '\n'.join(comment))
-        node = CommentNode()
-        node.value = '\n'.join(comment)
+
+        node.value = ''.join(comment)
         return node
 
     def read_name(self, t):
-        return t.next_token().value
+        return t.read_name()
 
     def read_dotted(self, t):
         tok = t.next_token()
@@ -225,6 +230,14 @@ class ProtoParser(object):
             tok.value = tok.value.strip("'" + '"')
         return tok
 
+    def read_string_or_dotted(self, t):
+        tok = t.readahead()
+        if tok.type == TOKEN_STRING and tok.value[0] in ['"', "'"]:
+            tok.value = tok.value.strip("'" + '"')
+            t.next_token()
+            return tok
+        return self.read_dotted(t)
+
     def read_equals(self, t):
         tok = t.next_token()
         self.assert_token_expression(tok.value == '=', tok, "Expected =, Got %s" % tok)
@@ -267,7 +280,6 @@ class ProtoParser(object):
         pe.col = token.start[1]
         raise pe
 
-
     def assert_token(self, token, type, value=None, message=None):
         message = "Expected %s %s" % (value, self.format_token_position(token))
 
@@ -325,7 +337,6 @@ class ProtoParser(object):
         self.assert_read(t, TOKEN_OP, ')')
         return node
 
-
     def parse_option_stmt(self, t):
         """
             option = 'option' optionBody ';';
@@ -354,15 +365,6 @@ class ProtoParser(object):
         logger.debug('Read option %s', node)
         return node
 
-    def parse_optional_stmt(self, t):
-        node = OptionalNode()
-        node.name = self.read_name(t)
-        node.type = self.read_name(t)
-        self.read_equals(t)
-        node.value = self.read_number(t).value
-        self.read_semi(t)
-        return node
-
     def parse_message_stmt(self, t):
         """
             message ='message' ident messageBody;
@@ -384,7 +386,10 @@ class ProtoParser(object):
                 node = self.parse_field_desc(t)
                 node.parent = mnode
                 mnode.add_child(node)
-
+            elif self.check_names(token, ['option']):
+                node = self.parse_option_stmt(t)
+                node.parent = mnode
+                mnode.add_child(node)
             elif self.check_single_line_comment(token):
                 node = self.read_single_line_comment(t)
                 mnode.add_child(node)
@@ -418,9 +423,18 @@ class ProtoParser(object):
             extension = intLit ( 'to' ( intLit | 'max' ) )?;
         """
         ext = ExtensionsNode()
-        ext.start = t.next_token().value
-        totoken = t.next_token()
-        ext.end = t.next_token().value
+        ext.start = int(t.next_token().value)
+
+        # check if a to exists
+        totoken = t.readahead()
+        if totoken.value == 'to':
+            totoken = t.next_token()
+            next = t.next_token().value
+            if next == 'max':
+                ext.end = -1
+            else:
+                ext.end = int(next)
+
         self.read_semi(t)
         return ext
 
@@ -448,8 +462,10 @@ class ProtoParser(object):
                 t.skip()
                 break
             if token.type == TOKEN_SPACE:
-
                 t.skip(line=1 if token.value == "\n" else 0)
+            elif self.check_names(token, ["option"]):
+                t.next_token()
+                enum.add_child(self.parse_option_stmt(t))
             elif self.check_single_line_comment(token):
                 node = self.read_single_line_comment(t)
                 enum.add_child(node)
@@ -475,8 +491,11 @@ class ProtoParser(object):
             extend ='extend' userType '{' ( field | group | ';' )* '}';
         """
         extend = ExtendNode()
-        extend.name = t.next_token().value
+        extend.name = t.read_id()
+
+        # skip the open brace
         token = t.next_token()
+
         while t.has_next():
             token = t.next_token()
             if token.type == TOKEN_OP and token.value == '}':
@@ -487,7 +506,10 @@ class ProtoParser(object):
             elif self.check_multiline_line_comment(t, token):
                 self.protonode.add_child(self.read_multi_line_comment(t))
             elif token.value == 'optional':
-                extend.add_child(self.parse_optional_stmt(t))
+                t.backtrack()
+                extend.add_child(self.parse_field_desc(t))
+            elif token.value in ["\n"]:
+                pass
             else:
                 self.raise_token_error(token, "Unexpected token: %s" % token)
         return extend
@@ -602,10 +624,8 @@ class ProtoParser(object):
         #next = t.next_token()
         logger.debug('NEXT IS %s', next)
         self.read_equals(t)
-        value = self.read_dotted(t)
-
-        return ''.join(name), value.value
 
 
-#tokens, remainder = scanner.scan(s)
-#print tokens, remainder[0:20]
+        value = self.read_string_or_dotted(t)
+
+        return ''.join(name).strip("\"'"), value.value
