@@ -4,9 +4,33 @@ from cStringIO import StringIO
 class Token(object):
     type = None
     value = None
+
     def __repr__(self):
         return str((self.type, self.value))
-        
+
+
+class Identifier(object):
+    value = ''
+
+    def __init__(self, value, line=None, column=None):
+        self.value = value
+        self.line = line
+        self.column = column
+
+    def __hash__(self):
+        return hash(self.value)
+
+    def __eq__(self, other):
+        if isinstance(other, basestring):
+            return self.value == other
+        elif isinstance(other, Identifier):
+            return self.value == other.value
+
+        return id(self) == id(other)
+
+    def __str__(self):
+        return self.value
+
 class Tokenizer(object):
     def __init__(self, data):
         self.line = 0
@@ -20,6 +44,8 @@ class Tokenizer(object):
         
     def read_id(self):
         name = []
+        line, column = self.line, self.column
+
         while self.has_next():            
             token = self.next_token(False)   
             if token.type == 1:
@@ -31,8 +57,12 @@ class Tokenizer(object):
             else:
                 self.backtrack()
                 break
-        return ''.join(name)                
-      
+
+        return Identifier(''.join(name), line, column)
+
+    def read_name(self):
+        return Identifier(self.next_token().value, self.line, self.column)
+
     def backtrack(self):            
         self.index -= 1
         
@@ -41,6 +71,8 @@ class Tokenizer(object):
         t = Token()
         t.type = toknum
         t.value = tokval
+        t.start = a
+        t.end = b
         t.line = 1
         return t
         
